import csv

import pandas as pd

from pyupurs.stateless_file_ops import count_rows, count_cols_per_row


def audit_extra_separators(file_path: str, delimiter: str = "|", number_of_columns: int = None) -> pd.DataFrame:
    """
    Report rows where there is

    :param file_path:
    :param delimiter:
    :param number_of_columns:
    :return:
    """
    n_rows = count_rows(file_path, delimiter=delimiter)
    rows_n_cols = count_cols_per_row(file_path, delimiter=delimiter)
    if not number_of_columns:
        # Estimate of the number of cols. Assuming the most frequent col count is a valid criterion.
        number_of_columns = pd.Series(rows_n_cols).value_counts().idxmax()

    list_rows_n_cols = rows_n_cols.values()

    n_col_occurrences = len(set(list_rows_n_cols))

    abnormal_rows = []
    # Engage in defective row extraction only if the column count is suspicious
    if n_col_occurrences > 1:
        with open(file_path, "r") as file:
            f_object = csv.reader(file, delimiter=delimiter)
            for row in f_object:
                n_cols_for_this_row = len(row)
                #
                if n_cols_for_this_row != number_of_columns:
                    abnormal_rows.append(row)

    defective_df = pd.DataFrame(data=abnormal_rows)
    return defective_df



import csv


def count_cols_per_row(file_path: str, delimiter=",") -> dict:
    """
    In a structured delimiter-separated file, count the number of columns per row.
    Note : This function operates without loading the file in memory.

    :param file_path: Path of the file to process.
    :param delimiter: Delimiter used within the file
    :return: A dictionary for which the keys are the index of the row, and the value are its number of columns.
    """
    with open(file_path, "r") as file:
        f_obj = csv.reader(file, delimiter=delimiter)
        col_count = [len(f) for f in f_obj]
        dict_col_count = {i: list_v for i, list_v in enumerate(col_count)}

        return dict_col_count


def count_rows(file_path: str, delimiter="|", count_header=True) -> int:
    """
    Count the number of rows in a structured separated file.
    Note : This function operates without loading the file in memory.

    :param file_path:
    :param delimiter:
    :param count_header:
    :return: The number of rows.
    """
    with open(file_path, "r") as file:
        f_obj = csv.reader(file, delimiter=delimiter)
        row_count = sum(1 for row in f_obj)
        if not count_header:
            row_count -= 1

        return row_count



import setuptools

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

setuptools.setup(
    name='pyupurs',
    version='0.0.3',
    author='Imad Youbi Idrissi',
    author_email='imad.youbi.idrissi@gmail.com',
    description="PYthon Utilitary Processing, Useful and Reliable Stuff",
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    url="https://gitlab.com/ImadYIdrissi/pyupurs",
    packages=setuptools.find_packages(),
    license='LICENSE.txt',
    classifiers=(
        "Programming Language :: Python :: 3.6",
        "Operating System :: Unix",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)"
    ),
    install_requires=requirements,
    python_requires='>=3.6'
    # scripts=['some_script.py', 'bin/some_other_script.py'],
)
